# ______________________________________________________________________________
# ******************************************************************************
#
# The simplest robot task: Just go and reach a point
#
# ______________________________________________________________________________
# ******************************************************************************


#-----------------------------------------------------------------------------
#SET THE PATH TO THE URDF AND MESHES
#Define robotName, urdfpath and initialConfig
dt = 5e-3
urdfPath = "~/git/trac/hrp2/hrp2_14_description/urdf/hrp2_14.urdf"
urdfDir = ["~/git/trac/hrp2"]
robotName = 'HRP2LAAS'
OperationalPointsMap = {'left-wrist'  : 'LARM_JOINT5',
                         'right-wrist' : 'RARM_JOINT5',
                         'left-ankle'  : 'LLEG_JOINT5',
                         'right-ankle' : 'RLEG_JOINT5',
                         'gaze'        : 'HEAD_JOINT1',
                         'waist'       : 'root_joint',
                         'chest'       : 'CHEST_JOINT1'}
initialConfig = (0., 0., 0.648702, 0., 0. , 0.,                  # Free flyer
                  0., 0., 0., 0.,                                 # Chest and head
                  0.261799, 0.17453, 0., -0.523599, 0., 0., 0.1, # Left Arm
                  #0.,0.,0.,0.,0.,                                # Left hand
                  0.261799, -0.17453,  0., -0.523599, 0., 0., 0.1, # Right Arm
                  #0.,0.,0.,0.,0.,                                # Right Hand
                  0., 0., -0.453786, 0.872665, -0.418879, 0.,     # Left Leg
                  0., 0., -0.453786, 0.872665, -0.418879, 0.      # Right Leg
              )


#-----------------------------------------------------------------------------
#---- ROBOT SPECIFICATIONS----------------------------------------------------
#-----------------------------------------------------------------------------


#-----------------------------------------------------------------------------
#---- DYN --------------------------------------------------------------------
#-----------------------------------------------------------------------------
from pinocchio.robot_wrapper import RobotWrapper
import pinocchio as se3
from dynamic_graph.sot.dynamics.humanoid_robot import HumanoidRobot

pinocchioRobot = RobotWrapper(urdfPath, urdfDir, se3.JointModelFreeFlyer())
pinocchioRobot.initDisplay(loadModel=True)
pinocchioRobot.display(pinocchioRobot.q0)


robot = HumanoidRobot(robotName, pinocchioRobot.model,
                      pinocchioRobot.data, initialConfig, OperationalPointsMap)


# ------------------------------------------------------------------------------
# ---- Kinematic Stack of Tasks (SoT)  -----------------------------------------
# ------------------------------------------------------------------------------
from dynamic_graph import plug
from dynamic_graph.sot.core import SOT
sot = SOT('sot')
sot.setSize(robot.dynamic.getDimension())
plug(sot.control,robot.device.control)



# ------------------------------------------------------------------------------
# ---- TASKS -------------------------------------------------------------------

# ---- TASK GRIP
from dynamic_graph.sot.core.meta_tasks_kine import MetaTaskKine6d, MetaTaskKineCom, gotoNd
from dynamic_graph.sot.core.matrix_util import matrixToTuple
from numpy import eye

taskRH    = MetaTaskKine6d('rh',robot.dynamic,'rh','RARM_JOINT5')
handMgrip = eye(4); handMgrip[0:3,3] = (0.1,0,0)
taskRH.opmodif = matrixToTuple(handMgrip)
taskRH.feature.frame('desired')
# --- STATIC COM (if not walking)
taskCom = MetaTaskKineCom(robot.dynamic)
robot.dynamic.com.recompute(0)
taskCom.featureDes.errorIN.value = robot.dynamic.com.value
taskCom.task.controlGain.value = 10


# --- CONTACTS
#define contactLF and contactRF
for name,joint in [ ['LF',OperationalPointsMap['left-ankle']], ['RF',OperationalPointsMap['right-ankle'] ] ]:
    contact = MetaTaskKine6d('contact'+name,robot.dynamic,name,joint)
    contact.feature.frame('desired')
    contact.gain.setConstant(10)
    contact.keep()
    locals()['contact'+name] = contact


target = (0.5,-0.2,1.3)

#addRobotViewer(robot, small=False)
#robot.viewer.updateElementConfig('zmp',target+(0,0,0))



gotoNd(taskRH,target,'111',(4.9,0.9,0.01,0.9))
sot.push(contactRF.task.name)
sot.push(contactLF.task.name)
sot.push(taskCom.task.name)
sot.push(taskRH.task.name)

#-------------------------------------------------------------------------------
#----- MAIN LOOP ---------------------------------------------------------------
#-------------------------------------------------------------------------------

from dynamic_graph.sot.dynamics import fromSotToPinocchio

def runner(n):
    for i in xrange(n):
        robot.device.increment(dt)
        pinocchioRobot.display(fromSotToPinocchio(robot.device.state.value))

#runner(1000)
